import QtQuick 1.1
//import com.nokia.symbian 1.1
import com.nokia.meego 1.0
import QtMobility.location 1.2
import com.nokia.extras 1.1



PageStackWindow {
    id: window
    showStatusBar: false
    initialPage: MainPage {
        id: theMainPage
        tools: toolBarLayout
        onSearch: {
            searchPhotos();

        }
        onSatellite: {
            setMapToType("satellite")
        }
        onStreet: {
            setMapToType("street")
        }
        onHybrid: {
            setMapToType("hybrid")
        }
        onPanMap: {
            panMyMap(lat, lng)
        }
        onActiveFocusChanged: {
            console.log("main page active")
        }

    }
   // initialPage: splashPage
    //showStatusBar: false
    showToolBar: true
    property Coordinate currentPos: myPositionSource.position.coordinate
    property Coordinate topLeft
    property Coordinate bottomRight
    property double currentLat
    property double currentLng
    property double currentCenterLat: myPositionSource.position.coordinate.latitude
    property double currentCenterLng: myPositionSource.position.coordinate.longitude
    property int currentMouseX
    property int currentMouseY
    property bool loading: false
    property int photosfound: 0
    property int  mapObjectsCreated: 0
    property bool flickrSearched: false
    property bool panoramioSearched: false
    property bool locationFoundViaGeocode: false
    property string userLocation
    property int timeInsecs: 0

  //  property string mapType: "street"
    //property mapType maptype: map.StreetMap
    Component.onCompleted: {
        myPositionSource.start();
        console.log()
        window.state = "mapShown"
        //locationQueryDialog.open()
        gpsTimer.start()

    }
    Timer{
        id: gpsTimer
        running: true
        interval: 1000
        repeat: true
        onTriggered: {
            timeInsecs ++
            console.log(timeInsecs)
            var lat = myPositionSource.position.coordinate.latitude
            if (lat){
                gpsTimer.stop()
                //locationQueryDialog.open()
            }
           else if (!lat && timeInsecs > 45){
                locationQueryDialog.open()
                gpsTimer.stop()
            }
        }

    }

    QueryDialog{
        id: locationQueryDialog
        titleText: "Info:"
        icon: "images/Marker.png"
        message: qsTr("GPS is not responding. Would you like to enter your location manually?")
        acceptButtonText: "Yes"
        rejectButtonText: "No"
        onAccepted: window.pageStack.push(locationPage)
        onRejected: locationQueryDialog.close()
        //height: 60


    }

    ToolBarLayout {
        id: toolBarLayout
        ToolIcon  {
            id: toolbarBack            
            platformIconId : "toolbar-back"
            onClicked: window.pageStack.depth <= 1 ? Qt.quit() : window.pageStack.pop()
        }
        ToolIcon {
            id: toolbarSearch            
            iconId : "toolbar-search"
            onClicked: {
                theMainPage.search();
            }
        }
        ToolIcon {
            id: toolbarLocation

            iconId: "toolbar-edit"
            onClicked: {
                window.pageStack.push(locationPage)
            }
        }
        ToolIcon {
            id: toolbarFavorites           
            iconId: "toolbar-favorite-mark"
            onClicked: {
                loadFavorites()
                window.pageStack.push(favoritesGrid)
            }
        }



    }

    Rectangle{
        id: pageToolbar
    height: 60
    //color: "#15add4"
    color: "steelblue"
    width: parent.width
    anchors.top: parent.top
    BusyIndicator{
        id: busyIndicator
        height: 30
        width: 30
        running: true
        anchors.left: parent.left
        anchors.verticalCenter: parent.verticalCenter
        visible: window.loading || (window.mapObjectsCreated < window.photosfound)
        //visible: photoModel.count && photoModel.count <=100
        platformStyle: BusyIndicatorStyle {
            size: "small"
            inverted: true
        }
    }

    Text {
        id: name
        text: qsTr("GeoSnaps")
        font.pixelSize: 33
        color: "white"
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.leftMargin: 10
        visible: window.pageStack.depth <= 1 ? false: true
    }



    ButtonRow{
        anchors.centerIn: parent
        visible: window.pageStack.depth <= 1 ? true: false
        height: parent.height * 0.8
        anchors.verticalCenter: parent.verticalCenter
        width: parent.width * 0.75
        Button{
            id: maptab
            text: "Map"
           // pressed: true
            onClicked: {
                theMainPage.street()
                window.state = "mapShown"
            }
        }
        Button{
            id: satelliteTab
            text: "Satellite"
            onClicked: {
                theMainPage.satellite()
                window.state = "satelliteShown"
            }
        }
        Button{
            id: terrainTab
            text: "Terrain"
             onClicked: {
                 theMainPage.hybrid()
                 window.state = "terrainShown"
             }
        }
    }



    ToolIcon{
        id: navigator
        iconId: "toolbar-list"
        anchors.right: parent.right
        anchors.verticalCenter: parent.verticalCenter
         visible: false
       // onClicked: window.pageStack.depth<=2?window.pageStack.push(photoList):window.pageStack.push(photogrid)
        onClicked: {
            window.pageStack.push(photoList)
        }


    }
    ToolIcon{
        id: gridToggler
        iconId: "toolbar-pages-all"
        anchors.right: parent.right
        anchors.verticalCenter: parent.verticalCenter
        visible: false
        onClicked: {
            window.pageStack.push(photogrid)
        }
    }

    }


    ListViewPage{
        id: photoList
        tools: toolBarLayout
    }
    PhotoGridView{
           id: photogrid
           tools: toolBarLayout
       }
    Details{
        id: photoDetails
        tools: toolBarLayout

    }
    Splash{
        id: splashPage
    }
    Favorites{
        id: favoritesGrid
        tools: toolBarLayout
    }
    AddLocation{
        id: locationPage
        tools: toolBarLayout
    }
    Carousel{
        id: theCarousel
        tools: toolBarLayout
    }

    ListModel{
        id: photoModel
    }
    ListModel{
        id: favoritesModel
    }
    InfoBanner{
        id: mybanner
        text: "Please wait....."
       // timeout: 1000
        //onClicked: mybanner.close()
    }
    InfoBanner{
        id: notifier

        //timeout: 3000
    }
    ContextMenu{
        id: photoContextmenu
        property string photo
        MenuLayout{
            MenuItem{
                text: "Add to Favorites"
                onClicked: {
                    console.log(photoContextmenu.photo.title)
                    //savePhoto(photoContextmenu.photo)
                }
            }
        }
    }


    PositionSource{
        id: myPositionSource
        updateInterval: 1000
        active: true
      onPositionChanged: {
          // do something to the map.
        //  console.log(window.currentPos.latitude)
          theMainPage.panMyMap()

      }
      onUpdateIntervalChanged: {
       //   console.log(window.currentPos.longitude)

      }
    }

    states: [
        State {
            name: "theCarouseActive"
            PropertyChanges {target: faveButton; visible: true}
           // when: favoritesGrid.status === PageStatus.Active
        },
        State{
            name: "photoGridActive"
            PropertyChanges{target: navigator; visible: true}
            when: photogrid.status === PageStatus.Active
        },
        State{
            name: "photoListActive"
            PropertyChanges{target: gridToggler; visible: true}
            when: photoList.status === PageStatus.Active
        },
        State{
            name: "homePageActive"
            PropertyChanges{target: navigator; visible: true}
            when: theMainPage.status === PageStatus.Active && (window.flickrSearched && window.panoramioSearched)
        },
        State{
            name: "mapShown"
            PropertyChanges{target: maptab; pressed: true}
        },
        State{
            name: "satelliteShown"
            PropertyChanges{target: satelliteTab; pressed: true}
        },
        State{
            name: "terrainShown"
            PropertyChanges{target: terrainTab; pressed: true}
        }


    ]


    //--------------------------some JavaScript logic glue --------------------

    function refreshMap(){
        /*
          get the current map bounding Box
          search for items based on that bounding box
*/
    }
    function dumpBoundingBox(){
        console.log(window.currentPos.latitude +"User Latitude")
        console.log(window.currentPos.longitude +"User Longitude")

    }
    function searchMapForPhotos(){
        /*
          This is the meat of the application. The main logic for searching photos goes here.
          we need these parameters:
          User location ???? latlng
          The BoundingBox----
          the main url constructed from the above

          1. Construct the url
          2. Get the photos via ajax
                a) start with panoramio
                b) then flickr
          3. Filter the received data structures accordingly
          4. Merge the both filtered data structures from flick and panoramio into a single array.
          5. build a landmarkmodel from the data
          6. populate the map with the in memory landmarkmodel
          The photos dictionary should have this metadata
          {
          "Title": Photo title,
          "Source": flickr or panoramio?,
          "Description": if available,
          "Date": date photo was taken,
          "Thumbnail": url to the photo's thumbnail location,
          "Url": The url of the main photo,
          "Coordinate": {"Latitude":xxx, "Longitude":xxx},
          ""
            }
            7. Buck Fitches


> def get_panoramio_photos(minx, miny, maxx, maxy):
    #url = http://www.panoramio.com/map/get_panoramas.php?set=public&from=0&to=20&minx=-180&miny=-90&maxx=180&maxy=90&size=medium&mapfilter=true
    url = "http://www.panoramio.com/map/get_panoramas.php?set=public&from=0&to=50&size=medium&mapfilter=true&"
    url2 = "http://www.panoramio.com/map/get_panoramas.php?set=public&from=0&to=20&minx=-180&miny=-90&maxx=180&maxy=90&size=medium&mapfilter=true"
    import json
    params = {
        "minx": minx,
        "miny": miny,
        "maxx": maxx,
        "maxy": maxy
        }
    encoded_params = urllib.urlencode(params)
    #new_url = url+encoded_params
    request = urllib2.Request(url, encoded_params)
    data = urllib2.urlopen(url2)
    response = data.read()
    info = json.loads(response)
    photos = info['photos']
    urls = [photo['photo_file_url'] for photo in photos]
    return urls

*/
    }


    function lastKnownLocation(){
        // we retrieve or save a user's last known location
    }
    function currentMapType(){
        return map.StreetMap
    }
    function searchPanoramioold(){
        var minx = window.topLeft.longitude
        var miny = window.bottomRight.latitude
        var maxx = window.bottomRight.longitude
        var maxy = window.topLeft.latitude
        var url = "http://www.panoramio.com/map/get_panoramas.php?set=public&from=0&to=50&minx="+minx+"&miny="+miny+"&maxx="+maxx+"&maxy="+maxy+"0&size=medium&mapfilter=true";
            console.log(url)
            var json = new XMLHttpRequest();
            //json.setRequestHeader("Content-Type", "text/plain;charset=UTF-8")
            json.onreadystatechange = function(){
            if(json.readyState == XMLHttpRequest.DONE){
               // if(json.readyState == 4){container.state = "homescreen"}
                try{
               var data = JSON.parse(json.responseText)
               // var data = eval('(' + json.responseText + ')');
                }
                catch(e){
                    //notifyUser("No "+ name +" found nearby")
                    console.log(json.responseText)
                }
                //var l = data.length
                var photos = data['photos']
                if(photos){
                    //console.log(photos)
                    var l = photos.length
                    for(var x=0; x<l; x++){
                        var url = photos[x]['photo_file_url']
                        var latitude = photos[x]['latitude']
                        var longitude = photos[x]['longitude']
                       // console.log(url)
                      //  console.log(longitude)
                       // console.log(latitude)
                      //  mapImage.Coordinate.latitude = latitude
                       // mapImage.Coordinate.longitude = longitude
                        map.addMapObject(icons)
                    }
                   // console.log(data)
                }
                else{
            //data fetch failed
                    console.log("data fetch failed")
                   // console.log(json.responseText)
            }
            }
            else{
                console.log(json.readyState)
                    //hide loading
                //container.state = "homescreen"

                    }
            }
            json.open("GET", url)
            json.send()
            //container.state = "loading"
           // resultsModel.clear()
        window.loading = true

    }
    function findUser(){
        busyIndicator.visible = true
        var lat = myPositionSource.position.coordinate.latitude
        var lng = myPositionSource.position.coordinate.longitude

    }
    function geocode(){

    }
    function sliceText(text, length){
        var newtext;
        if (text.length > length){
            newtext = text.slice(0, length)
            newtext += "..."
            return newtext
        }
        else{
            newtext = text
        }
        return newtext
    }
    function reverseGeocode(streetname){
        var url = "http://maps.googleapis.com/maps/api/geocode/json?address="+streetname+"&sensor=true";
        console.log(url)
        var json = new XMLHttpRequest();
        json.onreadystatechange = function(){
            if(json.readyState == XMLHttpRequest.DONE){
                if(json.readyState == 4){
                    try{
                var data = JSON.parse(json.responseText)
                    }catch(e){
                        //notifyUser("Sorry, network error")
                       // container.state = "govisible"
                        console.log("network error")
                        //notify user here tooo
                        return false
                    }
                    if (data['status'] == "OK") {
                       var  CURRENT_LAT = data["results"][0]["geometry"]["location"]["lat"];
                       var CURRENT_LNG = data["results"][0]["geometry"]["location"]["lng"];
                       var  CURRENT_LOCATION = data['results'][0]['formatted_address'];                      
                        console.log(CURRENT_LOCATION)                       
                        collectTheDamnGarbage()
                        window.pageStack.pop(null)
                        theMainPage.panMyMap(CURRENT_LAT,CURRENT_LNG)
                        window.currentCenterLat = CURRENT_LAT
                        window.currentCenterLng = CURRENT_LNG
                        window.loading = false
                        window.userLocation = CURRENT_LOCATION
                        window.locationFoundViaGeocode = true


                        }
                    else{
                        //Sorry didn't get that
                       // notifyUser("Sorry, didn't get that. Please try again.")
                        //container.state = "govisible"
                        console.log("there was an erroro while trying to geocode")
                        window.loading = false
                        // notify user here
                        return false
                    }
                }

            }
        }
        json.open("GET", url)
        json.send()
       // container.state = "geocodeloading"
        window.loading = true
    }
    function collectTheDamnGarbage(){
       // mainPage.markers = []
        window.mapObjectsCreated = 0
        window.photosfound = 0
        window.flickrSearched = false
        window.panoramioSearched = 0
    }
    function savePhoto(photo){

        var db = openDatabaseSync("photosdb", "1.0", "Photos Database", 1000000)
        db.transaction(
                    function(tx){
                        tx.executeSql('CREATE TABLE IF NOT EXISTS photosdb(title TEXT, url TEXT, latitude FLOAT, longitude FLOAT, source TEXT, thumbnail TEXT, upload_date TEXT, owner TEXT, photo_id TEXT)')
                        var item = tx.executeSql("SELECT * FROM photosdb WHERE photo_id = ?", [photo.photo_id])
                        var photoExists = item.rows.length
                        if(!photoExists){
                            tx.executeSql('INSERT INTO photosdb VALUES(?,?,?,?,?,?,?,?,?)',
                                         [photo.title,
                                          photo.url,
                                          photo.coordinate.latitude,
                                          photo.coordinate.longitude,
                                          photo.source,
                                          photo.thumbnail,
                                          photo.upload_date,
                                          photo.owner,
                                          photo.photo_id
                                         ]);
                            console.log(photo.title +" has been added to favorites")
                            notifyUser(photo.title +" has been added to favorites")
                        } else{
                            console.log(photo.title +" already exists in favorites")
                            notifyUser(photo.title +" already exists in favorites")
                        }
                    }
                    )

    }
    function loadFavorites(){
        /*
        db.transaction(
                    function(tx){
                        try{
                      var items = tx.executeSql("SELECT * FROM favoritesdb")
                        }
                        catch(e){
                            notifyUser("You have not saved any place to favorites yet.")
                            console.log("no objects found")
                        }
                        var numberOfItems = items.rows.length
                        if(numberOfItems){
                            favoritesModel.clear()
                            //resultsscreen.text = "Favorites"
                            //container.state = "results"
                        for(var i=0; i<=numberOfItems; i++){
                            var lat = items.rows.item(i).lat
                            var lng = items.rows.item(i).lng
                            var distance = computeDistance(appWindow.currentLat, appWindow.currentlng, lat, lng )
                            var objectDetails = {
                                'name':  items.rows.item(i).objectname,
                                'street':  items.rows.item(i).address,
                                'contact':  items.rows.item(i).contact,
                                'distance':  distance,
                                'details': items.rows.item(i).url,
                                'lat': lat,
                                'lng': lng
                        }
                             favoritesModel.append({"objects":objectDetails})
                            //resultsModel
                        }

                        }
                        else{
                         notifyUser("You have not added items to favorites yet.")
                        }
                    }
title TEXT, url TEXT, latitude FLOAT, longitude FLOAT, source TEXT, thumbnail TEXT, upload_date TEXT, owner TEXT, photo_id TEXT
                    )

*/

        console.log("this is the favorites page")
        var db = openDatabaseSync("photosdb", "1.0", "Photos Database", 1000000)
        db.transaction(
                    function(tx){
                        try{
                            var photos = tx.executeSql("SELECT * FROM photosdb")
                        }
                        catch(e){
                            console.log("not photos in favorites")
                    }
                        var numberOfPhotos = photos.rows.length
                        if(numberOfPhotos){
                            console.log(numberOfPhotos)
                            favoritesModel.clear()
                            for( var i=0; i<numberOfPhotos; i++){
                                var photo = {
                                    "title": photos.rows.item(i).title,
                                    "url": photos.rows.item(i).url,
                                    "coordinate": {
                                        "latitude": photos.rows.item(i).latitude,
                                        "longitude":  photos.rows.item(i).longitude
                                    },
                                    "source":photos.rows.item(i).source,
                                    "thumbnail": photos.rows.item(i).thumbnail,
                                    "upload_date": photos.rows.item(i).upload_date,
                                    "owner": photos.rows.item(i).owner,
                                    "photo_id": photos.rows.item(i).photo_id

                                }
                                favoritesModel.append({"photo": photo})

                            }
                        }
                    }
                    )
    }
    function notifyUser(text){
    notifier.text = text
    notifier.timerShowTime = 3000;
    notifier.show()
    }
    function convertTimeStampToDate(unix){
        var date = new Date(unix*1000)
        var day = date.getDate()
        var month = date.getMonth()
        var year = date.getFullYear()
        var theDate = day+"/"+month+"/"+year
        return theDate
    }

}
